<?php

namespace HCM\FPA\Calculos\FolhaComplementar;

use HCM\FPA\Entity\FpatpcalcmovEntity;
use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use HCM\FPA\Entity\FpatipomovimentEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpatipoassoceveEntity;
use HCM\FPA\Calculos\Geral\MemoriaCalculo;

class CalculoDissidioIncidencia
{
    private $entityManager;
    private $hcmParameters;
    private $_calculo_folha;
    private $_nrTipoModalidadeCalculo;
    private $_alteracaoSalario;
    private $_eventom;
    
    // Constantes para facilitar a leitura
    const TIPO_CALCULO = 1;
    const TP_MOV_MENSAL = FpatipomovimentEntity::TP_MOV_MENSAL;
    
    public function __construct($entityManager, $hcmParameters, $tipoModalidadeCalculo, $calculoFolhaObj, $itecalcOndeDeveGerarEvento295, $alteracaoSalario, $eventom)
    {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->_alteracaoSalario = $alteracaoSalario;
        $this->_eventom = $eventom;
        $this->_nrTipoModalidadeCalculo = $tipoModalidadeCalculo;
        $this->_calculo_folha = $calculoFolhaObj;
        $this->_itens_calculo_principal = $itecalcOndeDeveGerarEvento295;
    }
    
    public function calculaDissidioIncidencia($dtInicio, $dtFim, $vinculoParaCalculo, $incidenciaParaSerCalculada)
    {
        $tp_calc_movs = [];
        $ocorrencia = 1;
        
        $alteracaoSalario = $this->_alteracaoSalario->carregaAlteracaoSalario(
            $dtFim, $vinculoParaCalculo->getNrvinculom(), 1
        );
        
        $dataBaseVazia = empty($this->_alteracaoSalario->getDataBaseDissidio());
        if($dataBaseVazia){
            return;
        }
        
        $dataBaseDissidioMenorQueDataInicioFormulario = $this->_alteracaoSalario->getDataBaseDissidio() < $dtInicio;
        if($dataBaseDissidioMenorQueDataInicioFormulario){
            return;
        }

        // Itera por cada mês entre a data de início e a data de fim
        for ($dataAtual = clone $dtInicio; $dataAtual < $dtFim; $dataAtual->modify('+1 month')) {
            $mesAtual = $dataAtual->format('m');
            $anoAtual = $dataAtual->format('Y');

            // Verifica a condição do mês e ano
            if ($this->mesAtualAnoAtualIgualDataFim($mesAtual, $anoAtual, $dtFim)) {
                continue;
            }

            // Inicializa o tipo de cálculo do movimento para o mês atual
            $tp_calc_movs[$mesAtual] = $this->criaTipoCalculoMovimento($dataAtual, $mesAtual, $ocorrencia);
        }

        // Processa os cálculos do dissídio
        if ($tp_calc_movs) {
            $this->executaGeracaoEventoDissidio($tp_calc_movs, $vinculoParaCalculo, $dtFim, $incidenciaParaSerCalculada);
        }
    }

    private function mesAtualAnoAtualIgualDataFim($mesAtual, $anoAtual, $dtFim)
    {
        return $mesAtual >= $dtFim->format('m') && $anoAtual == $dtFim->format('Y');
    }

    private function criaTipoCalculoMovimento($dataAtual, $mesAtual, $ocorrencia)
    {
        $tp_calc_mov = new FpatpcalcmovEntity($this->entityManager, $this->hcmParameters);
        $tp_calc_mov->retornaTipoCalculoMovimento(self::TIPO_CALCULO);

        $competencia = DateUtil::getDataDeString("01/{$mesAtual}/{$dataAtual->format('Y')}");
        $tp_calc_mov->setCompetencia($competencia);
        $tp_calc_mov->atribuiNrCalculoFolha(
            $this->hcmParameters->getNrorgTrabalho(),
            $this->_calculo_folha,
            $competencia,
            $this->_nrTipoModalidadeCalculo,
            $ocorrencia
        );

        return $tp_calc_mov;
    }

    private function executaGeracaoEventoDissidio($tp_calc_movs, $vinculoParaCalculo, $dataLimite, $incidenciaParaSerCalculada)
    {
        // Obtém as informações de salário
        $salarioDissidio = $this->_alteracaoSalario->getSalario();
        $ultimasAlteSalarios = $this->_alteracaoSalario->retornaAlteSalarioUltimas(
            $dataLimite, $vinculoParaCalculo->getNrvinculom(), 1
        );

        // Calcula a variação percentual
        $percentualAltera = $this->calculaPercentualAlteracao($salarioDissidio, $ultimasAlteSalarios);

        // Processa os cálculos do dissídio
        $baseDissidio = $this->calculaBaseDissidio($tp_calc_movs, $vinculoParaCalculo, $dataLimite, $incidenciaParaSerCalculada);

        // Aplica o percentual ao total das incidências
        $this->geraEventoDissidio($baseDissidio, $percentualAltera, $dataLimite);
    }

    private function calculaPercentualAlteracao($salarioDissidio, $ultimasAlteSalarios)
    {
        $percentualAltera = 0;
        if (!empty($ultimasAlteSalarios)) {
            $salarioAnterior = end($ultimasAlteSalarios);
            $vrsalario_ante = $salarioAnterior->getVrsalario();

            if ($vrsalario_ante) {
                $percentualAltera = round((($salarioDissidio / $vrsalario_ante) - 1) * 100, 2);
            }
        }

        return $percentualAltera;
    }

    private function calculaBaseDissidio($tp_calc_movs, $vinculoParaCalculo, $dataLimite, $incidenciaParaSerCalculada)
    {
        $baseDissidio = 0;
        foreach ($tp_calc_movs as $tp_calc_mov) {
            $_mesCompetenciaAtual = DateUtil::getDataDeString(substr($tp_calc_mov->getCompetencia(), 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
            $calc_folhaAtual = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)
                ->retornaCalculoFolha(
                    $this->hcmParameters->getNrorgTrabalho(),
                    self::TP_MOV_MENSAL,
                    1, 1, $_mesCompetenciaAtual, $_mesCompetenciaAtual
                )[0];

            $itemCalcCompetenciaAtual = new FpaitecalcfolhaEntity($this->entityManager, $this->hcmParameters, $calc_folhaAtual->getNrcalculofolha());
            $itemCalcCompetenciaAtual->retornaItensPorVinculo(
                $vinculoParaCalculo->getNrorg(),
                $vinculoParaCalculo->getNrvinculom(),
                $calc_folhaAtual->getNrcalculofolha()
            );

            // Se a base de dissídio não existir ou for menor que a data base, pula o mês
            if (!$itemCalcCompetenciaAtual || $_mesCompetenciaAtual < $this->_alteracaoSalario->getDataBaseDissidio()) {
                continue;
            }

            // Adiciona a incidência ao total da base de dissídio
            $baseDissidio += $itemCalcCompetenciaAtual->calculaIncidencias(
                $incidenciaParaSerCalculada,
                $dataLimite,
                $this->_eventom
            );
        }
        
        return $baseDissidio;
    }

    private function geraEventoDissidio($baseDissidio, $percentualAltera, $dataLimite)
    {
        $evento295Dissidio = $this->_eventom->retornaEvento(FpatipoassoceveEntity::ASSOC_DIF_SALARIO_DISSIDIO);
        $valorCalculado295 = ($baseDissidio * $percentualAltera) / 100;

        // Se o valor calculado for maior que zero, gera o evento 295
        if ($valorCalculado295 > 0) {
            $this->_itens_calculo_principal->geraEvento($evento295Dissidio, null, $valorCalculado295, $percentualAltera, 1);
        }
    }
}